#ifndef INVERT_MATRIX_HPP
#define INVERT_MATRIX_HPP

/*
	Code from the boost Wiki:
	http://www.crystalclearsoftware.com/cgi-bin/boost_wiki/wiki.pl?BOOST_WIKI
*/

// REMEMBER to update "lu.hpp" header includes from boost-CVS
#include <boost/numeric/ublas/vector.hpp>
#include <boost/numeric/ublas/vector_proxy.hpp>
#include <boost/numeric/ublas/matrix.hpp>
#include <boost/numeric/ublas/triangular.hpp>
#include <boost/numeric/ublas/lu.hpp>
#include <boost/numeric/ublas/io.hpp>

#include <iostream>

namespace ublas = boost::numeric::ublas;

/* Matrix inversion routine.
	Uses lu_factorize and lu_substitute in uBLAS to invert a matrix */
template<class T>
bool invert(const ublas::matrix<T>& input, ublas::matrix<T>& inverse)
{
	using namespace boost::numeric::ublas;
	typedef permutation_matrix<std::size_t> pmatrix;
	// create a working copy of the input
	matrix<T> A(input);
	// create a permutation matrix for the LU-factorization
	pmatrix pm(A.size1());

	// perform LU-factorization
	int res = lu_factorize(A,pm);
	if( res != 0 )
		return false;
		
	/*std::cout << "m\n" << A << std::endl;
	std::cout << "pm\n" << pm << std::endl;*/

	// create identity matrix of "inverse"
	inverse.assign(ublas::identity_matrix<T>(A.size1()));

	// backsubstitute to get the inverse
	lu_substitute(A, pm, inverse);

	return true;
}

#endif //INVERT_MATRIX_HPP
