/*
	Fix Your Singer
	(c) 2008--2009 Stephane Magnenat <stephane at magnenat dot net>

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation version 3.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	The GPL can be found at <http://www.gnu.org/licenses/>.
*/


#include "MainWindow.h"
#include "PitchVector.h"
#include "PitchEditorWidget.h"
#include "Version.h"
#include <QtGui>
#include <QProcess>

MainWindow::MainWindow()
{
	editor = new PitchEditorWidget(&pitches, this);
	
	QMenu* fileMenu = menuBar()->addMenu(tr("&File"));
	fileMenu->addAction(QIcon(), tr("&Open"), this, SLOT(open()), QKeySequence(QKeySequence::Open));
	saveAsAction = fileMenu->addAction(QIcon(), tr("Save &As"), this, SLOT(saveAs()));
	saveAsAction->setEnabled(false);
	saveAction = fileMenu->addAction(QIcon(), tr("&Save"), this, SLOT(save()), QKeySequence(QKeySequence::Save));
	saveAction->setEnabled(false);
	fileMenu->addAction(QIcon(), tr("Export pitch tiers"), this, SLOT(exportPitchTiers()));
	
	fileMenu->addSeparator();
	
	playSoundAction = fileMenu->addAction(QIcon(), tr("&Play"), this, SLOT(play()), QKeySequence("Tab"));
	playSoundAction->setEnabled(false);
	connect(editor, SIGNAL(selectionStateChanged(bool)), playSoundAction, SLOT(setEnabled(bool)));
	// disabled because praat does not allows us to stop in script.. bug? did I miss something?
	/*stopPlayingSoundAction = fileMenu->addAction(QIcon(), tr("S&top playing sound"), this, SLOT(stopPlayingSound()), QKeySequence("Esc"));
	stopPlayingSoundAction->setEnabled(false);*/
	
	QMenu* editMenu = menuBar()->addMenu(tr("&Tools"));
	editMenu->addAction(QIcon(), tr("&Constant"), editor, SLOT(selectToolConstant()), QKeySequence("CTRL+c"));
	editMenu->addAction(QIcon(), tr("&Linear"), editor, SLOT(selectToolLinear()), QKeySequence("CTRL+l"));
	editMenu->addAction(QIcon(), tr("S&pline"), editor, SLOT(selectToolSpline()), QKeySequence("CTRL+p"));
	editMenu->addAction(QIcon(), tr("&Reset"), editor, SLOT(toolReset()), QKeySequence("CTRL+r"));
	
	QMenu* viewMenu = menuBar()->addMenu(tr("&View"));
	QAction* setScaleLatin = viewMenu->addAction(QIcon(), tr("&Latin scale"), editor, SLOT(setScaleLatin()));
	setScaleLatin->setCheckable(true);
	QAction* setScaleAnglosaxon = viewMenu->addAction(QIcon(), tr("&Anglosaxon scale"), editor, SLOT(setScaleAnglosaxon()));
	setScaleAnglosaxon->setCheckable(true);
	QActionGroup* scaleModeGroup = new QActionGroup(this);
	scaleModeGroup->addAction(setScaleLatin);
	scaleModeGroup->addAction(setScaleAnglosaxon);
	setScaleLatin->setChecked(true);
	
	setCentralWidget(editor);
	setWindowTitle(tr("Fix Your Singer - version "  FYS_VERSION));
	resize(600, 400);
	
	QString program = "/usr/bin/praat";
	QStringList arguments;
	arguments << "-";
	praat = new QProcess(this);
	praat->start(program, arguments);
}

MainWindow::~MainWindow()
{
	praat->write("Quit\n");
	praat->waitForFinished();
}

void MainWindow::open()
{
	//QString fileName = QFileDialog::getOpenFileName(this, tr("Open File"), "", tr("Praat pitch file (*.PitchTier)"));
	QString fileName = QFileDialog::getOpenFileName(this, tr("Open File"), "", tr("Wav sound (*.wav)"));
	if (!fileName.isEmpty())
	{
		QFile file(fileName);
		if (file.open(QIODevice::ReadOnly))
		{
			file.close();
			
			/*Read from file... /tmp/space in name . wav
			To Manipulation... 0.01 75 600
			Extract pitch tier
			Write to text file... /home/nct/untitled.PitchTier
			Remove
			select Manipulation space_in_name_
			Remove
			select Sound space_in_name_
			Remove*/
			
			QString lockFileName = QString ("%0/.fys.praat.lock").arg(getenv("HOME"));
			QString pitchFileName = QString ("%0/.fys.praat.PitchTier").arg(getenv("HOME"));
			
			// if we have a file already, remove all related data
			if (!openPraatFileName.isEmpty())
				praat->write("Select All\nRemove\n");
			
			// make praat open this file
			praat->write(QString("Read from file... %0\n").arg(fileName).toLocal8Bit());
			
			openFileName = fileName;
			openPraatFileName = fileName;
			openPraatFileName.replace(' ', '_');
			if (openPraatFileName.endsWith(".wav"))
				openPraatFileName.chop(4);
			int slashPos = openPraatFileName.lastIndexOf('/');
			if (slashPos >= 0)
				openPraatFileName = openPraatFileName.mid(slashPos+1);
			qDebug() << openPraatFileName;
			
			// extract pitch inside praat
			praat->write("To Manipulation... 0.01 75 600\n");
			praat->write("Extract pitch tier\n");
			praat->write(QString("Write to PitchTier spreadsheet file... %0\n").arg(pitchFileName).toLocal8Bit());
			
			praat->write(QString("system_nocheck touch %0\n").arg(lockFileName).toLocal8Bit());
			
			praat->waitForBytesWritten();
			
			qDebug() << "Waiting";
			while (true)
			{
				QFile lockFile(lockFileName);
				if (lockFile.exists())
					break;
				usleep(10000);
			}
			praat->write(QString("system_nocheck rm %0\n").arg(lockFileName).toLocal8Bit());
			praat->waitForBytesWritten();
			qDebug() << "Done";
			
			QFile pitchFile(pitchFileName);
			if (pitchFile.open(QIODevice::ReadOnly))
			{
				QTextStream in(&pitchFile);
				bool result = pitches.open(in);
				if (result)
				{
					saveAction->setEnabled(true);
					saveAsAction->setEnabled(true);
					editor->zoom(pitches.bounds);
				}
				else
					qDebug("error loading file");
			}
		}
	}
}

void MainWindow::replacePitchTier()
{
	QString pitchFileName = QString ("%0/.fys.out.PitchTier").arg(getenv("HOME"));
	if (savePitchTiers(pitchFileName))
	{
		praat->write(QString("Read from file... %0\n").arg(pitchFileName).toLocal8Bit());
		praat->write("select PitchTier _fys_out\n");
		praat->write(QString("plus Manipulation %0\n").arg(openPraatFileName).toLocal8Bit());
		praat->write("Replace pitch tier\n");
		praat->write("select PitchTier _fys_out\n");
		praat->write("Remove\n");
	}
}

void MainWindow::saveAs()
{
	QString fileName = QFileDialog::getSaveFileName(this, tr("Save File"), openFileName, tr("Wav sound (*.wav)"));
	if (doSave(fileName))
		saveFileName = fileName;
}

void MainWindow::save()
{
	if (!doSave(saveFileName))
		saveAs();
}

void MainWindow::exportPitchTiers()
{
	QString fileName = QFileDialog::getSaveFileName(this, tr("Export pitch tiers"), openFileName, tr("Praat pitch tiers (*.PitchTier)"));
	savePitchTiers(fileName);
}

bool MainWindow::doSave(const QString& fileName)
{
	if (!fileName.isEmpty())
	{
		replacePitchTier();
		praat->write(QString("select Manipulation %0\n").arg(openPraatFileName).toLocal8Bit());
		praat->write("Get resynthesis (overlap-add)\n");
		praat->write(QString("Write to WAV file... %0\n").arg(fileName).toLocal8Bit());
		praat->write("Remove\n");
		praat->waitForBytesWritten();
		return true;
	}
	return false;
}

bool MainWindow::savePitchTiers(const QString& fileName)
{
	QFile file(fileName);
	if (file.open(QIODevice::WriteOnly))
	{
		QTextStream out(&file);
		pitches.save(out);
		return true;
	}
	return false;
}

void MainWindow::play()
{
	if (!openPraatFileName.isEmpty())
	{
		replacePitchTier();
		
		// TODO: fix memory leaking in praat here 
		praat->write(QString("select Manipulation %0\n").arg(openPraatFileName).toLocal8Bit());
		praat->write("Get resynthesis (overlap-add)\n");
		praat->write(QString("Extract part... %0 %1 Rectangular 1 yes\n").arg(editor->getSelectionStart()).arg(editor->getSelectionStop()).toLocal8Bit());
		praat->write("Play\n");
		praat->write("Remove\n");
		praat->write("Remove\n");
		
		
		//praat->write("Play (overlap-add)\n");
		praat->waitForBytesWritten();
		
		//playSoundAction->setEnabled(false);
		//stopPlayingSoundAction->setEnabled(true);
	}
}

void MainWindow::stopPlayingSound()
{
	// TODO: does not work
	if (!openPraatFileName.isEmpty())
	{
		//praat->write("select Sound resynth\n");
		praat->write("Stop playing sound\n");
		//praat->write("Remove\n");
		praat->waitForBytesWritten();
		
		stopPlayingSoundAction->setEnabled(false);
		playSoundAction->setEnabled(true);
	}
}



