/*
	Fix Your Singer
	(c) 2008--2009 Stephane Magnenat <stephane at magnenat dot net>

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation version 3.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	The GPL can be found at <http://www.gnu.org/licenses/>.
*/


#ifndef __PITCH_EDITOR_WIDGET_H
#define __PITCH_EDITOR_WIDGET_H

#include <QWidget>
#include <boost/numeric/ublas/vector.hpp>

class PitchVector;

namespace ublas = boost::numeric::ublas;

class PitchEditorWidget : public QWidget
{
	Q_OBJECT
	
public:
	PitchEditorWidget(PitchVector* pitchData, QWidget* parent);
	void zoom(const QRectF& rect);

signals:
	void selectionStateChanged(bool ready);

public slots:
	void selectToolConstant();
	void selectToolLinear();
	void selectToolSpline();
	void toolReset();
	void setScaleLatin();
	void setScaleAnglosaxon();
	
	double getSelectionStart() const { return selectionStart; }
	double getSelectionStop() const { return selectionStop; }

protected:
	virtual void paintEvent ( QPaintEvent * event );
	virtual void mousePressEvent ( QMouseEvent * event );
	virtual void mouseMoveEvent ( QMouseEvent * event );
	virtual void mouseReleaseEvent ( QMouseEvent * event );
	virtual void wheelEvent ( QWheelEvent * event );
	
	void setupCurveControlParams(unsigned dof);
	double evaluateCurveControl(double pos);
	double evaluateCurveControl(double pos, const ublas::vector<double> &params);
	double evaluateCurveControlDer(double pos, const ublas::vector<double> &params);
	void getSplineCP(ublas::vector<double> &cpX, ublas::vector<double> &cpY);
	void setSplineFromCP(const ublas::vector<double> &cpX, const ublas::vector<double> &cpY, ublas::vector<double> &np);
	
	int freqToPixel(qreal freq);
	qreal pixelToFreq(int pos);
	int posToPixel(qreal pos);
	qreal pixelToPos(int pixel);
	
protected:
	PitchVector* pitchData;
	QRectF window;
	enum ScaleDisplay
	{
		SCALE_LATIN,
		SCALE_ANGLOSAXON
	} scaleDisplay;
	qreal selectionStart;
	qreal selectionStop;
	enum SelectionMode
	{
		SELECTION_NONE = 0,
		SELECTION_ONE_POINT,
		SELECTION_FULL
	} selectionMode;
	enum Tool
	{
		TOOL_NONE = 0,
		TOOL_CONSTANT,
		TOOL_LINEAR,
		TOOL_SPLINE
	} editTool;
	ublas::vector<double> curveControlParams;
	int dragParamIndex;
	QPoint dragParamPos;
	bool panning;
	QPoint panningPos;
};

#endif
