/*
	Fix Your Singer
	(c) 2008--2009 Stephane Magnenat <stephane at magnenat dot net>

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation version 3.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	The GPL can be found at <http://www.gnu.org/licenses/>.
*/


#include <PitchVector.h>
#include <QTextStream>
#include <QtGui>

PitchVector::PitchVector() :
	bounds(0, 100, 1, 400)
{
	
}

bool PitchVector::open(QTextStream& stream)
{
	QString fileType;
	stream >> fileType;
	if (fileType != "\"ooTextFile\"")
		return false;
	
	QString fileContent;
	stream >> fileContent;
	if (fileContent != "\"PitchTier\"")
		return false;
	
	float value;
	stream >> value;
	bounds.setLeft(value);
	stream >> value;
	bounds.setRight(value);
	
	int pointsCount;
	stream >> pointsCount;
	originalData.resize(pointsCount);
	editedData.resize(pointsCount);
	
	for (int i = 0; i < pointsCount; ++i)
	{
		QPointF p;
		stream >> p.rx();
		stream >> p.ry();
		if (i == 0)
		{
			bounds.setLeft(p.x());
			bounds.setWidth(1);
		}
		else if (i == pointsCount - 1)
		{
			bounds.setRight(p.x());
		}
		editedData[i] = originalData[i] = p;
	}
	
	// TODO: send signal
	
	return true;
}

void PitchVector::save(QTextStream& stream)
{
	stream << "\"ooTextFile\"" << endl;
	stream << "\"PitchTier\"" << endl;
	stream << bounds.left() << " " << bounds.right() << " " << editedData.size() << endl;
	for (int i = 0; i < editedData.size(); ++i)
	{
		stream << editedData[i].x() << " " << editedData[i].y() << endl;
	}
}

int PitchVector::getIndexFromTime(qreal time, bool right)
{
	if (originalData.size() == 0)
		return -1;
	
	// special case
	if (time <= originalData[0].x())
		return 0;
	else if (time >= originalData[originalData.size() - 1].x())
		return originalData.size() - 1;
	
	int h = originalData.size() - 1;
	int l = 0;
	while (h - l > 1)
	{
		int i = (h + l) / 2;
		double v = originalData[i].x();
		if (v == time)
			return i;
		else if (v < time)
			l = i;
		else
			h = i;
	}
	
	if (right)
		return h;
	else
		return l;
}
